# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from script import Gs2ScriptRestClient, request as request_, result as result_
from script.domain.iterator.namespaces import DescribeNamespacesIterator
from script.domain.iterator.scripts import DescribeScriptsIterator
from script.domain.cache.namespace import NamespaceDomainCache
from script.domain.cache.script import ScriptDomainCache


class ScriptDomain:
    _session: Gs2RestSession
    _client: Gs2ScriptRestClient
    _script_cache: ScriptDomainCache
    _namespace_name: str
    _script_name: str

    def __init__(
        self,
        session: Gs2RestSession,
        script_cache: ScriptDomainCache,
        namespace_name: str,
        script_name: str,
    ):
        self._session = session
        self._client = Gs2ScriptRestClient(
            session,
        )
        self._script_cache = script_cache
        self._namespace_name = namespace_name
        self._script_name = script_name

    def load(
        self,
        request: request_.GetScriptRequest,
    ) -> result_.GetScriptResult:
        request.with_namespace_name(self._namespace_name)
        request.with_script_name(self._script_name)
        r = self._client.get_script(
            request,
        )
        self._script_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateScriptRequest,
    ) -> result_.UpdateScriptResult:
        request.with_namespace_name(self._namespace_name)
        request.with_script_name(self._script_name)
        r = self._client.update_script(
            request,
        )
        self._script_cache.update(r.item)
        return r

    def update_from_git_hub(
        self,
        request: request_.UpdateScriptFromGitHubRequest,
    ) -> result_.UpdateScriptFromGitHubResult:
        request.with_namespace_name(self._namespace_name)
        request.with_script_name(self._script_name)
        r = self._client.update_script_from_git_hub(
            request,
        )
        self._script_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteScriptRequest,
    ) -> result_.DeleteScriptResult:
        request.with_namespace_name(self._namespace_name)
        request.with_script_name(self._script_name)
        r = self._client.delete_script(
            request,
        )
        return r



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from showcase import Gs2ShowcaseRestClient, request as request_, result as result_
from showcase.domain.iterator.namespaces import DescribeNamespacesIterator
from showcase.domain.iterator.sales_item_masters import DescribeSalesItemMastersIterator
from showcase.domain.iterator.sales_item_group_masters import DescribeSalesItemGroupMastersIterator
from showcase.domain.iterator.showcase_masters import DescribeShowcaseMastersIterator
from showcase.domain.iterator.showcases import DescribeShowcasesIterator
from showcase.domain.iterator.showcases_by_user_id import DescribeShowcasesByUserIdIterator
from showcase.domain.cache.namespace import NamespaceDomainCache
from showcase.domain.cache.sales_item_master import SalesItemMasterDomainCache
from showcase.domain.cache.sales_item_group_master import SalesItemGroupMasterDomainCache
from showcase.domain.cache.showcase_master import ShowcaseMasterDomainCache
from showcase.domain.cache.showcase import ShowcaseDomainCache
from showcase.domain.current_showcase_master import CurrentShowcaseMasterDomain
from showcase.domain.user import UserDomain
from showcase.domain.user_access_token import UserAccessTokenDomain
from showcase.domain.user_access_token import UserAccessTokenDomain
from showcase.domain.sales_item_master import SalesItemMasterDomain
from showcase.domain.sales_item_group_master import SalesItemGroupMasterDomain
from showcase.domain.showcase_master import ShowcaseMasterDomain


class NamespaceDomain:
    _session: Gs2RestSession
    _client: Gs2ShowcaseRestClient
    _namespace_cache: NamespaceDomainCache
    _namespace_name: str
    _sales_item_master_cache: SalesItemMasterDomainCache
    _sales_item_group_master_cache: SalesItemGroupMasterDomainCache
    _showcase_master_cache: ShowcaseMasterDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_cache: NamespaceDomainCache,
        namespace_name: str,
    ):
        self._session = session
        self._client = Gs2ShowcaseRestClient(
            session,
        )
        self._namespace_cache = namespace_cache
        self._namespace_name = namespace_name
        self._sales_item_master_cache = SalesItemMasterDomainCache()
        self._sales_item_group_master_cache = SalesItemGroupMasterDomainCache()
        self._showcase_master_cache = ShowcaseMasterDomainCache()

    def get_status(
        self,
        request: request_.GetNamespaceStatusRequest,
    ) -> result_.GetNamespaceStatusResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace_status(
            request,
        )
        return r

    def load(
        self,
        request: request_.GetNamespaceRequest,
    ) -> result_.GetNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateNamespaceRequest,
    ) -> result_.UpdateNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.update_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteNamespaceRequest,
    ) -> result_.DeleteNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.delete_namespace(
            request,
        )
        self._namespace_cache.delete(r.item)
        return r

    def create_sales_item_master(
        self,
        request: request_.CreateSalesItemMasterRequest,
    ) -> result_.CreateSalesItemMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_sales_item_master(
            request,
        )
        return r

    def create_sales_item_group_master(
        self,
        request: request_.CreateSalesItemGroupMasterRequest,
    ) -> result_.CreateSalesItemGroupMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_sales_item_group_master(
            request,
        )
        return r

    def create_showcase_master(
        self,
        request: request_.CreateShowcaseMasterRequest,
    ) -> result_.CreateShowcaseMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_showcase_master(
            request,
        )
        return r

    def sales_item_masters(
        self,
    ) -> DescribeSalesItemMastersIterator:
        return DescribeSalesItemMastersIterator(
            self._sales_item_master_cache,
            self._client,
            self._namespace_name,
        )

    def sales_item_group_masters(
        self,
    ) -> DescribeSalesItemGroupMastersIterator:
        return DescribeSalesItemGroupMastersIterator(
            self._sales_item_group_master_cache,
            self._client,
            self._namespace_name,
        )

    def showcase_masters(
        self,
    ) -> DescribeShowcaseMastersIterator:
        return DescribeShowcaseMastersIterator(
            self._showcase_master_cache,
            self._client,
            self._namespace_name,
        )

    def current_showcase_master(
        self,
    ) -> CurrentShowcaseMasterDomain:
        return CurrentShowcaseMasterDomain(
            self._session,
            self._namespace_name,
        )

    def user(
        self,
        user_id: str,
    ) -> UserDomain:
        return UserDomain(
            self._session,
            self._namespace_name,
            user_id,
        )

    def access_token(
        self,
        access_token: AccessToken,
    ) -> UserAccessTokenDomain:
        return UserAccessTokenDomain(
            self._session,
            self._namespace_name,
            access_token,
        )

    def sales_item_master(
        self,
        sales_item_name: str,
    ) -> SalesItemMasterDomain:
        return SalesItemMasterDomain(
            self._session,
            self._sales_item_master_cache,
            self._namespace_name,
            sales_item_name,
        )

    def sales_item_group_master(
        self,
        sales_item_group_name: str,
    ) -> SalesItemGroupMasterDomain:
        return SalesItemGroupMasterDomain(
            self._session,
            self._sales_item_group_master_cache,
            self._namespace_name,
            sales_item_group_name,
        )

    def showcase_master(
        self,
        showcase_name: str,
    ) -> ShowcaseMasterDomain:
        return ShowcaseMasterDomain(
            self._session,
            self._showcase_master_cache,
            self._namespace_name,
            showcase_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

import re
from typing import *
import core


class StatsEvent(core.Gs2Model):
    grn: str = None
    service: str = None
    method: str = None
    metric: str = None
    cumulative: bool = None
    value: float = None
    tags: List[str] = None
    call_at: int = None

    def with_grn(self, grn: str) -> StatsEvent:
        self.grn = grn
        return self

    def with_service(self, service: str) -> StatsEvent:
        self.service = service
        return self

    def with_method(self, method: str) -> StatsEvent:
        self.method = method
        return self

    def with_metric(self, metric: str) -> StatsEvent:
        self.metric = metric
        return self

    def with_cumulative(self, cumulative: bool) -> StatsEvent:
        self.cumulative = cumulative
        return self

    def with_value(self, value: float) -> StatsEvent:
        self.value = value
        return self

    def with_tags(self, tags: List[str]) -> StatsEvent:
        self.tags = tags
        return self

    def with_call_at(self, call_at: int) -> StatsEvent:
        self.call_at = call_at
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[StatsEvent]:
        if data is None:
            return None
        return StatsEvent()\
            .with_grn(data.get('grn'))\
            .with_service(data.get('service'))\
            .with_method(data.get('method'))\
            .with_metric(data.get('metric'))\
            .with_cumulative(data.get('cumulative'))\
            .with_value(data.get('value'))\
            .with_tags([
                data.get('tags')[i]
                for i in range(len(data.get('tags')) if data.get('tags') else 0)
            ])\
            .with_call_at(data.get('callAt'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "grn": self.grn,
            "service": self.service,
            "method": self.method,
            "metric": self.metric,
            "cumulative": self.cumulative,
            "value": self.value,
            "tags": [
                self.tags[i]
                for i in range(len(self.tags) if self.tags else 0)
            ],
            "callAt": self.call_at,
        }


class BillingActivity(core.Gs2Model):
    billing_activity_id: str = None
    year: int = None
    month: int = None
    service: str = None
    activity_type: str = None
    value: int = None

    def with_billing_activity_id(self, billing_activity_id: str) -> BillingActivity:
        self.billing_activity_id = billing_activity_id
        return self

    def with_year(self, year: int) -> BillingActivity:
        self.year = year
        return self

    def with_month(self, month: int) -> BillingActivity:
        self.month = month
        return self

    def with_service(self, service: str) -> BillingActivity:
        self.service = service
        return self

    def with_activity_type(self, activity_type: str) -> BillingActivity:
        self.activity_type = activity_type
        return self

    def with_value(self, value: int) -> BillingActivity:
        self.value = value
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        year,
        month,
        service,
        activity_type,
    ):
        return 'grn:gs2:{region}:{ownerId}:watch:{year}:{month}:{service}:{activityType}'.format(
            region=region,
            ownerId=owner_id,
            year=year,
            month=month,
            service=service,
            activityType=activity_type,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<year>.+):(?P<month>.+):(?P<service>.+):(?P<activityType>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<year>.+):(?P<month>.+):(?P<service>.+):(?P<activityType>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_year_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<year>.+):(?P<month>.+):(?P<service>.+):(?P<activityType>.+)', grn)
        if match is None:
            return None
        return match.group('year')

    @classmethod
    def get_month_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<year>.+):(?P<month>.+):(?P<service>.+):(?P<activityType>.+)', grn)
        if match is None:
            return None
        return match.group('month')

    @classmethod
    def get_service_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<year>.+):(?P<month>.+):(?P<service>.+):(?P<activityType>.+)', grn)
        if match is None:
            return None
        return match.group('service')

    @classmethod
    def get_activity_type_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<year>.+):(?P<month>.+):(?P<service>.+):(?P<activityType>.+)', grn)
        if match is None:
            return None
        return match.group('activity_type')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[BillingActivity]:
        if data is None:
            return None
        return BillingActivity()\
            .with_billing_activity_id(data.get('billingActivityId'))\
            .with_year(data.get('year'))\
            .with_month(data.get('month'))\
            .with_service(data.get('service'))\
            .with_activity_type(data.get('activityType'))\
            .with_value(data.get('value'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "billingActivityId": self.billing_activity_id,
            "year": self.year,
            "month": self.month,
            "service": self.service,
            "activityType": self.activity_type,
            "value": self.value,
        }


class Cumulative(core.Gs2Model):
    cumulative_id: str = None
    resource_grn: str = None
    name: str = None
    value: int = None
    updated_at: int = None

    def with_cumulative_id(self, cumulative_id: str) -> Cumulative:
        self.cumulative_id = cumulative_id
        return self

    def with_resource_grn(self, resource_grn: str) -> Cumulative:
        self.resource_grn = resource_grn
        return self

    def with_name(self, name: str) -> Cumulative:
        self.name = name
        return self

    def with_value(self, value: int) -> Cumulative:
        self.value = value
        return self

    def with_updated_at(self, updated_at: int) -> Cumulative:
        self.updated_at = updated_at
        return self

    @classmethod
    def create_grn(
        cls,
        region,
        owner_id,
        resource_grn,
        name,
    ):
        return 'grn:gs2:{region}:{ownerId}:watch:{resourceGrn}:{name}'.format(
            region=region,
            ownerId=owner_id,
            resourceGrn=resource_grn,
            name=name,
        )

    @classmethod
    def get_region_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<resourceGrn>.+):(?P<name>.+)', grn)
        if match is None:
            return None
        return match.group('region')

    @classmethod
    def get_owner_id_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<resourceGrn>.+):(?P<name>.+)', grn)
        if match is None:
            return None
        return match.group('owner_id')

    @classmethod
    def get_resource_grn_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<resourceGrn>.+):(?P<name>.+)', grn)
        if match is None:
            return None
        return match.group('resource_grn')

    @classmethod
    def get_name_from_grn(
        cls,
        grn: str,
    ) -> Optional[str]:
        match = re.search('grn:gs2:(?P<region>.+):(?P<ownerId>.+):watch:(?P<resourceGrn>.+):(?P<name>.+)', grn)
        if match is None:
            return None
        return match.group('name')

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Cumulative]:
        if data is None:
            return None
        return Cumulative()\
            .with_cumulative_id(data.get('cumulativeId'))\
            .with_resource_grn(data.get('resourceGrn'))\
            .with_name(data.get('name'))\
            .with_value(data.get('value'))\
            .with_updated_at(data.get('updatedAt'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "cumulativeId": self.cumulative_id,
            "resourceGrn": self.resource_grn,
            "name": self.name,
            "value": self.value,
            "updatedAt": self.updated_at,
        }


class Chart(core.Gs2Model):
    chart_id: str = None
    embed_id: str = None
    html: str = None

    def with_chart_id(self, chart_id: str) -> Chart:
        self.chart_id = chart_id
        return self

    def with_embed_id(self, embed_id: str) -> Chart:
        self.embed_id = embed_id
        return self

    def with_html(self, html: str) -> Chart:
        self.html = html
        return self

    @classmethod
    def create_grn(
        cls,
    ):
        return ''.format(
        )

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[Chart]:
        if data is None:
            return None
        return Chart()\
            .with_chart_id(data.get('chartId'))\
            .with_embed_id(data.get('embedId'))\
            .with_html(data.get('html'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "chartId": self.chart_id,
            "embedId": self.embed_id,
            "html": self.html,
        }


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from mission.domain.service import *
from mission.domain.complete import *
from mission.domain.complete_access_token import *
from mission.domain.complete import *
from mission.domain.complete_access_token import *
from mission.domain.counter_model_master import *
from mission.domain.mission_group_model_master import *
from mission.domain.namespace import *
from mission.domain.counter import *
from mission.domain.counter_access_token import *
from mission.domain.counter import *
from mission.domain.counter_access_token import *
from mission.domain.current_mission_master import *
from mission.domain.counter_model import *
from mission.domain.mission_group_model import *
from mission.domain.mission_task_model import *
from mission.domain.mission_task_model_master import *
from mission.domain.user import *
from mission.domain.user_access_token import *
from mission.domain.user import *
from mission.domain.user_access_token import *
from mission.domain.iterator.completes import *
from mission.domain.iterator.completes_by_user_id import *
from mission.domain.iterator.counter_model_masters import *
from mission.domain.iterator.mission_group_model_masters import *
from mission.domain.iterator.namespaces import *
from mission.domain.iterator.counters import *
from mission.domain.iterator.counters_by_user_id import *
from mission.domain.iterator.counter_models import *
from mission.domain.iterator.mission_group_models import *
from mission.domain.iterator.mission_task_models import *
from mission.domain.iterator.mission_task_model_masters import *
from mission.domain.cache.complete import *
from mission.domain.cache.counter_model_master import *
from mission.domain.cache.mission_group_model_master import *
from mission.domain.cache.namespace import *
from mission.domain.cache.counter import *
from mission.domain.cache.counter_model import *
from mission.domain.cache.mission_group_model import *
from mission.domain.cache.mission_task_model import *
from mission.domain.cache.mission_task_model_master import *


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from __future__ import annotations

from watch.model import *


class GetChartRequest(core.Gs2Request):

    context_stack: str = None
    metrics: str = None
    grn: str = None
    queries: List[str] = None
    by: str = None
    timeframe: str = None
    size: str = None
    format: str = None
    aggregator: str = None
    style: str = None
    title: str = None

    def with_metrics(self, metrics: str) -> GetChartRequest:
        self.metrics = metrics
        return self

    def with_grn(self, grn: str) -> GetChartRequest:
        self.grn = grn
        return self

    def with_queries(self, queries: List[str]) -> GetChartRequest:
        self.queries = queries
        return self

    def with_by(self, by: str) -> GetChartRequest:
        self.by = by
        return self

    def with_timeframe(self, timeframe: str) -> GetChartRequest:
        self.timeframe = timeframe
        return self

    def with_size(self, size: str) -> GetChartRequest:
        self.size = size
        return self

    def with_format(self, format: str) -> GetChartRequest:
        self.format = format
        return self

    def with_aggregator(self, aggregator: str) -> GetChartRequest:
        self.aggregator = aggregator
        return self

    def with_style(self, style: str) -> GetChartRequest:
        self.style = style
        return self

    def with_title(self, title: str) -> GetChartRequest:
        self.title = title
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetChartRequest]:
        if data is None:
            return None
        return GetChartRequest()\
            .with_metrics(data.get('metrics'))\
            .with_grn(data.get('grn'))\
            .with_queries([
                data.get('queries')[i]
                for i in range(len(data.get('queries')) if data.get('queries') else 0)
            ])\
            .with_by(data.get('by'))\
            .with_timeframe(data.get('timeframe'))\
            .with_size(data.get('size'))\
            .with_format(data.get('format'))\
            .with_aggregator(data.get('aggregator'))\
            .with_style(data.get('style'))\
            .with_title(data.get('title'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "metrics": self.metrics,
            "grn": self.grn,
            "queries": [
                self.queries[i]
                for i in range(len(self.queries) if self.queries else 0)
            ],
            "by": self.by,
            "timeframe": self.timeframe,
            "size": self.size,
            "format": self.format,
            "aggregator": self.aggregator,
            "style": self.style,
            "title": self.title,
        }


class GetCumulativeRequest(core.Gs2Request):

    context_stack: str = None
    name: str = None
    resource_grn: str = None

    def with_name(self, name: str) -> GetCumulativeRequest:
        self.name = name
        return self

    def with_resource_grn(self, resource_grn: str) -> GetCumulativeRequest:
        self.resource_grn = resource_grn
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetCumulativeRequest]:
        if data is None:
            return None
        return GetCumulativeRequest()\
            .with_name(data.get('name'))\
            .with_resource_grn(data.get('resourceGrn'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self.name,
            "resourceGrn": self.resource_grn,
        }


class DescribeBillingActivitiesRequest(core.Gs2Request):

    context_stack: str = None
    year: int = None
    month: int = None
    service: str = None
    page_token: str = None
    limit: int = None

    def with_year(self, year: int) -> DescribeBillingActivitiesRequest:
        self.year = year
        return self

    def with_month(self, month: int) -> DescribeBillingActivitiesRequest:
        self.month = month
        return self

    def with_service(self, service: str) -> DescribeBillingActivitiesRequest:
        self.service = service
        return self

    def with_page_token(self, page_token: str) -> DescribeBillingActivitiesRequest:
        self.page_token = page_token
        return self

    def with_limit(self, limit: int) -> DescribeBillingActivitiesRequest:
        self.limit = limit
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[DescribeBillingActivitiesRequest]:
        if data is None:
            return None
        return DescribeBillingActivitiesRequest()\
            .with_year(data.get('year'))\
            .with_month(data.get('month'))\
            .with_service(data.get('service'))\
            .with_page_token(data.get('pageToken'))\
            .with_limit(data.get('limit'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "year": self.year,
            "month": self.month,
            "service": self.service,
            "pageToken": self.page_token,
            "limit": self.limit,
        }


class GetBillingActivityRequest(core.Gs2Request):

    context_stack: str = None
    year: int = None
    month: int = None
    service: str = None
    activity_type: str = None

    def with_year(self, year: int) -> GetBillingActivityRequest:
        self.year = year
        return self

    def with_month(self, month: int) -> GetBillingActivityRequest:
        self.month = month
        return self

    def with_service(self, service: str) -> GetBillingActivityRequest:
        self.service = service
        return self

    def with_activity_type(self, activity_type: str) -> GetBillingActivityRequest:
        self.activity_type = activity_type
        return self

    def get(self, key, default=None):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return default

    def __getitem__(self, key):
        items = self.to_dict()
        if key in items.keys():
            return items[key]
        return None

    @staticmethod
    def from_dict(
        data: Dict[str, Any],
    ) -> Optional[GetBillingActivityRequest]:
        if data is None:
            return None
        return GetBillingActivityRequest()\
            .with_year(data.get('year'))\
            .with_month(data.get('month'))\
            .with_service(data.get('service'))\
            .with_activity_type(data.get('activityType'))

    def to_dict(self) -> Dict[str, Any]:
        return {
            "year": self.year,
            "month": self.month,
            "service": self.service,
            "activityType": self.activity_type,
        }


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from formation import Gs2FormationRestClient, request as request_, result as result_
from formation.domain.iterator.namespaces import DescribeNamespacesIterator
from formation.domain.iterator.form_model_masters import DescribeFormModelMastersIterator
from formation.domain.iterator.mold_models import DescribeMoldModelsIterator
from formation.domain.iterator.mold_model_masters import DescribeMoldModelMastersIterator
from formation.domain.iterator.molds import DescribeMoldsIterator
from formation.domain.iterator.molds_by_user_id import DescribeMoldsByUserIdIterator
from formation.domain.iterator.forms import DescribeFormsIterator
from formation.domain.iterator.forms_by_user_id import DescribeFormsByUserIdIterator
from formation.domain.cache.namespace import NamespaceDomainCache
from formation.domain.cache.form_model_master import FormModelMasterDomainCache
from formation.domain.cache.mold_model import MoldModelDomainCache
from formation.domain.cache.mold_model_master import MoldModelMasterDomainCache
from formation.domain.cache.mold import MoldDomainCache
from formation.domain.cache.form import FormDomainCache
from formation.domain.mold import MoldDomain
from formation.domain.mold_access_token import MoldAccessTokenDomain
from formation.domain.mold_access_token import MoldAccessTokenDomain


class UserDomain:
    _session: Gs2RestSession
    _client: Gs2FormationRestClient
    _namespace_name: str
    _user_id: str
    _mold_cache: MoldDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_name: str,
        user_id: str,
    ):
        self._session = session
        self._client = Gs2FormationRestClient(
            session,
        )
        self._namespace_name = namespace_name
        self._user_id = user_id
        self._mold_cache = MoldDomainCache()

    def molds(
        self,
    ) -> DescribeMoldsByUserIdIterator:
        return DescribeMoldsByUserIdIterator(
            self._mold_cache,
            self._client,
            self._namespace_name,
            self._user_id,
        )

    def mold(
        self,
        mold_name: str,
    ) -> MoldDomain:
        return MoldDomain(
            self._session,
            self._mold_cache,
            self._namespace_name,
            self._user_id,
            mold_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from lottery import Gs2LotteryRestClient, request as request_, result as result_
from lottery.domain.iterator.namespaces import DescribeNamespacesIterator
from lottery.domain.iterator.lottery_model_masters import DescribeLotteryModelMastersIterator
from lottery.domain.iterator.prize_table_masters import DescribePrizeTableMastersIterator
from lottery.domain.iterator.boxes import DescribeBoxesIterator
from lottery.domain.iterator.boxes_by_user_id import DescribeBoxesByUserIdIterator
from lottery.domain.iterator.lottery_models import DescribeLotteryModelsIterator
from lottery.domain.iterator.prize_tables import DescribePrizeTablesIterator
from lottery.domain.iterator.probabilities import DescribeProbabilitiesIterator
from lottery.domain.iterator.probabilities_by_user_id import DescribeProbabilitiesByUserIdIterator
from lottery.domain.cache.namespace import NamespaceDomainCache
from lottery.domain.cache.lottery_model_master import LotteryModelMasterDomainCache
from lottery.domain.cache.prize_table_master import PrizeTableMasterDomainCache
from lottery.domain.cache.box import BoxDomainCache
from lottery.domain.cache.lottery_model import LotteryModelDomainCache
from lottery.domain.cache.prize_table import PrizeTableDomainCache
from lottery.domain.cache.probability import ProbabilityDomainCache
from lottery.domain.user import UserDomain
from lottery.domain.user_access_token import UserAccessTokenDomain
from lottery.domain.user_access_token import UserAccessTokenDomain
from lottery.domain.current_lottery_master import CurrentLotteryMasterDomain
from lottery.domain.lottery_model import LotteryModelDomain
from lottery.domain.prize_table_master import PrizeTableMasterDomain
from lottery.domain.lottery_model_master import LotteryModelMasterDomain
from lottery.domain.prize_table import PrizeTableDomain


class NamespaceDomain:
    _session: Gs2RestSession
    _client: Gs2LotteryRestClient
    _namespace_cache: NamespaceDomainCache
    _namespace_name: str
    _lottery_model_master_cache: LotteryModelMasterDomainCache
    _prize_table_master_cache: PrizeTableMasterDomainCache
    _lottery_model_cache: LotteryModelDomainCache
    _prize_table_cache: PrizeTableDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_cache: NamespaceDomainCache,
        namespace_name: str,
    ):
        self._session = session
        self._client = Gs2LotteryRestClient(
            session,
        )
        self._namespace_cache = namespace_cache
        self._namespace_name = namespace_name
        self._lottery_model_master_cache = LotteryModelMasterDomainCache()
        self._prize_table_master_cache = PrizeTableMasterDomainCache()
        self._lottery_model_cache = LotteryModelDomainCache()
        self._prize_table_cache = PrizeTableDomainCache()

    def get_status(
        self,
        request: request_.GetNamespaceStatusRequest,
    ) -> result_.GetNamespaceStatusResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace_status(
            request,
        )
        return r

    def load(
        self,
        request: request_.GetNamespaceRequest,
    ) -> result_.GetNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateNamespaceRequest,
    ) -> result_.UpdateNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.update_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteNamespaceRequest,
    ) -> result_.DeleteNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.delete_namespace(
            request,
        )
        self._namespace_cache.delete(r.item)
        return r

    def create_lottery_model_master(
        self,
        request: request_.CreateLotteryModelMasterRequest,
    ) -> result_.CreateLotteryModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_lottery_model_master(
            request,
        )
        return r

    def create_prize_table_master(
        self,
        request: request_.CreatePrizeTableMasterRequest,
    ) -> result_.CreatePrizeTableMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_prize_table_master(
            request,
        )
        return r

    def lottery_model_masters(
        self,
    ) -> DescribeLotteryModelMastersIterator:
        return DescribeLotteryModelMastersIterator(
            self._lottery_model_master_cache,
            self._client,
            self._namespace_name,
        )

    def prize_table_masters(
        self,
    ) -> DescribePrizeTableMastersIterator:
        return DescribePrizeTableMastersIterator(
            self._prize_table_master_cache,
            self._client,
            self._namespace_name,
        )

    def lottery_models(
        self,
    ) -> DescribeLotteryModelsIterator:
        return DescribeLotteryModelsIterator(
            self._lottery_model_cache,
            self._client,
            self._namespace_name,
        )

    def prize_tables(
        self,
    ) -> DescribePrizeTablesIterator:
        return DescribePrizeTablesIterator(
            self._prize_table_cache,
            self._client,
            self._namespace_name,
        )

    def user(
        self,
        user_id: str,
    ) -> UserDomain:
        return UserDomain(
            self._session,
            self._namespace_name,
            user_id,
        )

    def access_token(
        self,
        access_token: AccessToken,
    ) -> UserAccessTokenDomain:
        return UserAccessTokenDomain(
            self._session,
            self._namespace_name,
            access_token,
        )

    def current_lottery_master(
        self,
    ) -> CurrentLotteryMasterDomain:
        return CurrentLotteryMasterDomain(
            self._session,
            self._namespace_name,
        )

    def lottery_model(
        self,
        lottery_name: str,
    ) -> LotteryModelDomain:
        return LotteryModelDomain(
            self._session,
            self._lottery_model_cache,
            self._namespace_name,
            lottery_name,
        )

    def prize_table_master(
        self,
        prize_table_name: str,
    ) -> PrizeTableMasterDomain:
        return PrizeTableMasterDomain(
            self._session,
            self._prize_table_master_cache,
            self._namespace_name,
            prize_table_name,
        )

    def lottery_model_master(
        self,
        lottery_name: str,
    ) -> LotteryModelMasterDomain:
        return LotteryModelMasterDomain(
            self._session,
            self._lottery_model_master_cache,
            self._namespace_name,
            lottery_name,
        )

    def prize_table(
        self,
        prize_table_name: str,
    ) -> PrizeTableDomain:
        return PrizeTableDomain(
            self._session,
            self._prize_table_cache,
            self._namespace_name,
            prize_table_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from chat import Gs2ChatRestClient, request as request_, result as result_
from chat.domain.iterator.namespaces import DescribeNamespacesIterator
from chat.domain.iterator.rooms import DescribeRoomsIterator
from chat.domain.iterator.messages import DescribeMessagesIterator
from chat.domain.iterator.messages_by_user_id import DescribeMessagesByUserIdIterator
from chat.domain.iterator.subscribes import DescribeSubscribesIterator
from chat.domain.iterator.subscribes_by_user_id import DescribeSubscribesByUserIdIterator
from chat.domain.iterator.subscribes_by_room_name import DescribeSubscribesByRoomNameIterator
from chat.domain.cache.namespace import NamespaceDomainCache
from chat.domain.cache.room import RoomDomainCache
from chat.domain.cache.message import MessageDomainCache
from chat.domain.cache.subscribe import SubscribeDomainCache
from chat.domain.message import MessageDomain
from chat.domain.message_access_token import MessageAccessTokenDomain
from chat.domain.message_access_token import MessageAccessTokenDomain


class RoomAccessTokenDomain:
    _session: Gs2RestSession
    _client: Gs2ChatRestClient
    _room_cache: RoomDomainCache
    _namespace_name: str
    _access_token: AccessToken
    _room_name: str
    _password: str
    _message_cache: MessageDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        room_cache: RoomDomainCache,
        namespace_name: str,
        access_token: AccessToken,
        room_name: str,
        password: str,
    ):
        self._session = session
        self._client = Gs2ChatRestClient(
            session,
        )
        self._room_cache = room_cache
        self._namespace_name = namespace_name
        self._access_token = access_token
        self._room_name = room_name
        self._password = password
        self._message_cache = MessageDomainCache()

    def update(
        self,
        request: request_.UpdateRoomRequest,
    ) -> result_.UpdateRoomResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        request.with_room_name(self._room_name)
        request.with_password(self._password)
        r = self._client.update_room(
            request,
        )
        self._room_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteRoomRequest,
    ) -> result_.DeleteRoomResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        request.with_room_name(self._room_name)
        r = self._client.delete_room(
            request,
        )
        self._room_cache.delete(r.item)
        return r

    def post(
        self,
        request: request_.PostRequest,
    ) -> result_.PostResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        request.with_room_name(self._room_name)
        request.with_password(self._password)
        r = self._client.post(
            request,
        )
        return r

    def messages(
        self,
    ) -> DescribeMessagesIterator:
        return DescribeMessagesIterator(
            self._message_cache,
            self._client,
            self._namespace_name,
            self._room_name,
            self._password,
            self._access_token,
        )

    def message(
        self,
        message_name: str,
    ) -> MessageAccessTokenDomain:
        return MessageAccessTokenDomain(
            self._session,
            self._message_cache,
            self._namespace_name,
            self._access_token,
            self._room_name,
            message_name,
            self._password,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from quest import Gs2QuestRestClient, request as request_, result as result_
from quest.domain.iterator.namespaces import DescribeNamespacesIterator
from quest.domain.iterator.quest_group_model_masters import DescribeQuestGroupModelMastersIterator
from quest.domain.iterator.quest_model_masters import DescribeQuestModelMastersIterator
from quest.domain.iterator.progresses_by_user_id import DescribeProgressesByUserIdIterator
from quest.domain.iterator.completed_quest_lists import DescribeCompletedQuestListsIterator
from quest.domain.iterator.completed_quest_lists_by_user_id import DescribeCompletedQuestListsByUserIdIterator
from quest.domain.iterator.quest_group_models import DescribeQuestGroupModelsIterator
from quest.domain.iterator.quest_models import DescribeQuestModelsIterator
from quest.domain.cache.namespace import NamespaceDomainCache
from quest.domain.cache.quest_group_model_master import QuestGroupModelMasterDomainCache
from quest.domain.cache.quest_model_master import QuestModelMasterDomainCache
from quest.domain.cache.progress import ProgressDomainCache
from quest.domain.cache.completed_quest_list import CompletedQuestListDomainCache
from quest.domain.cache.quest_group_model import QuestGroupModelDomainCache
from quest.domain.cache.quest_model import QuestModelDomainCache


class ProgressAccessTokenDomain:
    _session: Gs2RestSession
    _client: Gs2QuestRestClient
    _progress_cache: ProgressDomainCache
    _namespace_name: str
    _access_token: AccessToken

    def __init__(
        self,
        session: Gs2RestSession,
        progress_cache: ProgressDomainCache,
        namespace_name: str,
        access_token: AccessToken,
    ):
        self._session = session
        self._client = Gs2QuestRestClient(
            session,
        )
        self._progress_cache = progress_cache
        self._namespace_name = namespace_name
        self._access_token = access_token

    def load(
        self,
        request: request_.GetProgressRequest,
    ) -> result_.GetProgressResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        r = self._client.get_progress(
            request,
        )
        self._progress_cache.update(r.item)
        return r

    def start(
        self,
        request: request_.StartRequest,
    ) -> result_.StartResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        r = self._client.start(
            request,
        )
        return r

    def end(
        self,
        request: request_.EndRequest,
    ) -> result_.EndResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        r = self._client.end(
            request,
        )
        self._progress_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteProgressRequest,
    ) -> result_.DeleteProgressResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        r = self._client.delete_progress(
            request,
        )
        self._progress_cache.delete(r.item)
        return r



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from mission import Gs2MissionRestClient, request as request_, result as result_
from mission.domain.iterator.completes import DescribeCompletesIterator
from mission.domain.iterator.completes_by_user_id import DescribeCompletesByUserIdIterator
from mission.domain.iterator.counter_model_masters import DescribeCounterModelMastersIterator
from mission.domain.iterator.mission_group_model_masters import DescribeMissionGroupModelMastersIterator
from mission.domain.iterator.namespaces import DescribeNamespacesIterator
from mission.domain.iterator.counters import DescribeCountersIterator
from mission.domain.iterator.counters_by_user_id import DescribeCountersByUserIdIterator
from mission.domain.iterator.counter_models import DescribeCounterModelsIterator
from mission.domain.iterator.mission_group_models import DescribeMissionGroupModelsIterator
from mission.domain.iterator.mission_task_models import DescribeMissionTaskModelsIterator
from mission.domain.iterator.mission_task_model_masters import DescribeMissionTaskModelMastersIterator
from mission.domain.cache.complete import CompleteDomainCache
from mission.domain.cache.counter_model_master import CounterModelMasterDomainCache
from mission.domain.cache.mission_group_model_master import MissionGroupModelMasterDomainCache
from mission.domain.cache.namespace import NamespaceDomainCache
from mission.domain.cache.counter import CounterDomainCache
from mission.domain.cache.counter_model import CounterModelDomainCache
from mission.domain.cache.mission_group_model import MissionGroupModelDomainCache
from mission.domain.cache.mission_task_model import MissionTaskModelDomainCache
from mission.domain.cache.mission_task_model_master import MissionTaskModelMasterDomainCache


class CompleteAccessTokenDomain:
    _session: Gs2RestSession
    _client: Gs2MissionRestClient
    _complete_cache: CompleteDomainCache
    _namespace_name: str
    _access_token: AccessToken
    _mission_group_name: str

    def __init__(
        self,
        session: Gs2RestSession,
        complete_cache: CompleteDomainCache,
        namespace_name: str,
        access_token: AccessToken,
        mission_group_name: str,
    ):
        self._session = session
        self._client = Gs2MissionRestClient(
            session,
        )
        self._complete_cache = complete_cache
        self._namespace_name = namespace_name
        self._access_token = access_token
        self._mission_group_name = mission_group_name

    def complete(
        self,
        request: request_.CompleteRequest,
    ) -> result_.CompleteResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        request.with_mission_group_name(self._mission_group_name)
        r = self._client.complete(
            request,
        )
        return r

    def load(
        self,
        request: request_.GetCompleteRequest,
    ) -> result_.GetCompleteResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        request.with_mission_group_name(self._mission_group_name)
        r = self._client.get_complete(
            request,
        )
        self._complete_cache.update(r.item)
        return r



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from identifier import Gs2IdentifierRestClient, request as request_, result as result_
from identifier.domain.iterator.users import DescribeUsersIterator
from identifier.domain.iterator.security_policies import DescribeSecurityPoliciesIterator
from identifier.domain.iterator.common_security_policies import DescribeCommonSecurityPoliciesIterator
from identifier.domain.iterator.identifiers import DescribeIdentifiersIterator
from identifier.domain.iterator.passwords import DescribePasswordsIterator
from identifier.domain.cache.user import UserDomainCache
from identifier.domain.cache.security_policy import SecurityPolicyDomainCache
from identifier.domain.cache.identifier import IdentifierDomainCache
from identifier.domain.cache.password import PasswordDomainCache


class SecurityPolicyDomain:
    _session: Gs2RestSession
    _client: Gs2IdentifierRestClient
    _security_policy_cache: SecurityPolicyDomainCache
    _security_policy_name: str

    def __init__(
        self,
        session: Gs2RestSession,
        security_policy_cache: SecurityPolicyDomainCache,
        security_policy_name: str,
    ):
        self._session = session
        self._client = Gs2IdentifierRestClient(
            session,
        )
        self._security_policy_cache = security_policy_cache
        self._security_policy_name = security_policy_name

    def load(
        self,
        request: request_.GetSecurityPolicyRequest,
    ) -> result_.GetSecurityPolicyResult:
        request.with_security_policy_name(self._security_policy_name)
        r = self._client.get_security_policy(
            request,
        )
        self._security_policy_cache.update(r.item)
        return r



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from formation import Gs2FormationRestClient, request as request_, result as result_
from formation.domain.iterator.namespaces import DescribeNamespacesIterator
from formation.domain.iterator.form_model_masters import DescribeFormModelMastersIterator
from formation.domain.iterator.mold_models import DescribeMoldModelsIterator
from formation.domain.iterator.mold_model_masters import DescribeMoldModelMastersIterator
from formation.domain.iterator.molds import DescribeMoldsIterator
from formation.domain.iterator.molds_by_user_id import DescribeMoldsByUserIdIterator
from formation.domain.iterator.forms import DescribeFormsIterator
from formation.domain.iterator.forms_by_user_id import DescribeFormsByUserIdIterator
from formation.domain.cache.namespace import NamespaceDomainCache
from formation.domain.cache.form_model_master import FormModelMasterDomainCache
from formation.domain.cache.mold_model import MoldModelDomainCache
from formation.domain.cache.mold_model_master import MoldModelMasterDomainCache
from formation.domain.cache.mold import MoldDomainCache
from formation.domain.cache.form import FormDomainCache
from formation.domain.current_form_master import CurrentFormMasterDomain
from formation.domain.mold_model import MoldModelDomain
from formation.domain.user import UserDomain
from formation.domain.user_access_token import UserAccessTokenDomain
from formation.domain.user_access_token import UserAccessTokenDomain
from formation.domain.form_model_master import FormModelMasterDomain
from formation.domain.mold_model_master import MoldModelMasterDomain


class NamespaceDomain:
    _session: Gs2RestSession
    _client: Gs2FormationRestClient
    _namespace_cache: NamespaceDomainCache
    _namespace_name: str
    _form_model_master_cache: FormModelMasterDomainCache
    _mold_model_cache: MoldModelDomainCache
    _mold_model_master_cache: MoldModelMasterDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_cache: NamespaceDomainCache,
        namespace_name: str,
    ):
        self._session = session
        self._client = Gs2FormationRestClient(
            session,
        )
        self._namespace_cache = namespace_cache
        self._namespace_name = namespace_name
        self._form_model_master_cache = FormModelMasterDomainCache()
        self._mold_model_cache = MoldModelDomainCache()
        self._mold_model_master_cache = MoldModelMasterDomainCache()

    def get_status(
        self,
        request: request_.GetNamespaceStatusRequest,
    ) -> result_.GetNamespaceStatusResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace_status(
            request,
        )
        return r

    def load(
        self,
        request: request_.GetNamespaceRequest,
    ) -> result_.GetNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateNamespaceRequest,
    ) -> result_.UpdateNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.update_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteNamespaceRequest,
    ) -> result_.DeleteNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.delete_namespace(
            request,
        )
        self._namespace_cache.delete(r.item)
        return r

    def create_form_model_master(
        self,
        request: request_.CreateFormModelMasterRequest,
    ) -> result_.CreateFormModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_form_model_master(
            request,
        )
        return r

    def create_mold_model_master(
        self,
        request: request_.CreateMoldModelMasterRequest,
    ) -> result_.CreateMoldModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_mold_model_master(
            request,
        )
        return r

    def form_model_masters(
        self,
    ) -> DescribeFormModelMastersIterator:
        return DescribeFormModelMastersIterator(
            self._form_model_master_cache,
            self._client,
            self._namespace_name,
        )

    def mold_models(
        self,
    ) -> DescribeMoldModelsIterator:
        return DescribeMoldModelsIterator(
            self._mold_model_cache,
            self._client,
            self._namespace_name,
        )

    def mold_model_masters(
        self,
    ) -> DescribeMoldModelMastersIterator:
        return DescribeMoldModelMastersIterator(
            self._mold_model_master_cache,
            self._client,
            self._namespace_name,
        )

    def current_form_master(
        self,
    ) -> CurrentFormMasterDomain:
        return CurrentFormMasterDomain(
            self._session,
            self._namespace_name,
        )

    def mold_model(
        self,
        mold_name: str,
    ) -> MoldModelDomain:
        return MoldModelDomain(
            self._session,
            self._mold_model_cache,
            self._namespace_name,
            mold_name,
        )

    def user(
        self,
        user_id: str,
    ) -> UserDomain:
        return UserDomain(
            self._session,
            self._namespace_name,
            user_id,
        )

    def access_token(
        self,
        access_token: AccessToken,
    ) -> UserAccessTokenDomain:
        return UserAccessTokenDomain(
            self._session,
            self._namespace_name,
            access_token,
        )

    def form_model_master(
        self,
        form_model_name: str,
    ) -> FormModelMasterDomain:
        return FormModelMasterDomain(
            self._session,
            self._form_model_master_cache,
            self._namespace_name,
            form_model_name,
        )

    def mold_model_master(
        self,
        mold_name: str,
    ) -> MoldModelMasterDomain:
        return MoldModelMasterDomain(
            self._session,
            self._mold_model_master_cache,
            self._namespace_name,
            mold_name,
        )



# encoding: utf-8
#
# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

from core.rest import *
from core.model import Gs2Constant
from watch.request import *
from watch.result import *


class Gs2WatchRestClient(AbstractGs2RestClient):

    def _get_chart(
        self,
        request: GetChartRequest,
        callback: Callable[[AsyncResult[GetChartResult]], None],
        is_blocking: bool,
    ):
        url = Gs2Constant.ENDPOINT_HOST.format(
            service='watch',
            region=self.session.region,
        ) + "/chart/{metrics}".format(
            metrics=request.metrics if request.metrics is not None and request.metrics != '' else 'null',
        )

        headers = self._create_authorized_headers()
        body = {
            'contextStack': request.context_stack,
        }
        if request.grn is not None:
            body["grn"] = request.grn
        if request.queries is not None:
            body["queries"] = [
                item
                for item in request.queries
            ]
        if request.by is not None:
            body["by"] = request.by
        if request.timeframe is not None:
            body["timeframe"] = request.timeframe
        if request.size is not None:
            body["size"] = request.size
        if request.format is not None:
            body["format"] = request.format
        if request.aggregator is not None:
            body["aggregator"] = request.aggregator
        if request.style is not None:
            body["style"] = request.style
        if request.title is not None:
            body["title"] = request.title

        if request.request_id:
            headers["X-GS2-REQUEST-ID"] = request.request_id
        _job = NetworkJob(
            url=url,
            method='POST',
            result_type=GetChartResult,
            callback=callback,
            headers=headers,
            body=body,
        )

        self.session.send(
            job=_job,
            is_blocking=is_blocking,
        )

    def get_chart(
        self,
        request: GetChartRequest,
    ) -> GetChartResult:
        async_result = []
        with timeout(30):
            self._get_chart(
                request,
                lambda result: async_result.append(result),
                is_blocking=True,
            )

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_chart_async(
        self,
        request: GetChartRequest,
    ) -> GetChartResult:
        async_result = []
        self._get_chart(
            request,
            lambda result: async_result.append(result),
            is_blocking=False,
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_cumulative(
        self,
        request: GetCumulativeRequest,
        callback: Callable[[AsyncResult[GetCumulativeResult]], None],
        is_blocking: bool,
    ):
        url = Gs2Constant.ENDPOINT_HOST.format(
            service='watch',
            region=self.session.region,
        ) + "/cumulative/{name}".format(
            name=request.name if request.name is not None and request.name != '' else 'null',
        )

        headers = self._create_authorized_headers()
        body = {
            'contextStack': request.context_stack,
        }
        if request.resource_grn is not None:
            body["resourceGrn"] = request.resource_grn

        if request.request_id:
            headers["X-GS2-REQUEST-ID"] = request.request_id
        _job = NetworkJob(
            url=url,
            method='POST',
            result_type=GetCumulativeResult,
            callback=callback,
            headers=headers,
            body=body,
        )

        self.session.send(
            job=_job,
            is_blocking=is_blocking,
        )

    def get_cumulative(
        self,
        request: GetCumulativeRequest,
    ) -> GetCumulativeResult:
        async_result = []
        with timeout(30):
            self._get_cumulative(
                request,
                lambda result: async_result.append(result),
                is_blocking=True,
            )

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_cumulative_async(
        self,
        request: GetCumulativeRequest,
    ) -> GetCumulativeResult:
        async_result = []
        self._get_cumulative(
            request,
            lambda result: async_result.append(result),
            is_blocking=False,
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_billing_activities(
        self,
        request: DescribeBillingActivitiesRequest,
        callback: Callable[[AsyncResult[DescribeBillingActivitiesResult]], None],
        is_blocking: bool,
    ):
        url = Gs2Constant.ENDPOINT_HOST.format(
            service='watch',
            region=self.session.region,
        ) + "/billingActivity/{year}/{month}".format(
            year=request.year if request.year is not None and request.year != '' else 'null',
            month=request.month if request.month is not None and request.month != '' else 'null',
        )

        headers = self._create_authorized_headers()
        query_strings = {
            'contextStack': request.context_stack,
        }
        if request.service is not None:
            query_strings["service"] = request.service
        if request.page_token is not None:
            query_strings["pageToken"] = request.page_token
        if request.limit is not None:
            query_strings["limit"] = request.limit

        if request.request_id:
            headers["X-GS2-REQUEST-ID"] = request.request_id
        _job = NetworkJob(
            url=url,
            method='GET',
            result_type=DescribeBillingActivitiesResult,
            callback=callback,
            headers=headers,
            query_strings=query_strings,
        )

        self.session.send(
            job=_job,
            is_blocking=is_blocking,
        )

    def describe_billing_activities(
        self,
        request: DescribeBillingActivitiesRequest,
    ) -> DescribeBillingActivitiesResult:
        async_result = []
        with timeout(30):
            self._describe_billing_activities(
                request,
                lambda result: async_result.append(result),
                is_blocking=True,
            )

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_billing_activities_async(
        self,
        request: DescribeBillingActivitiesRequest,
    ) -> DescribeBillingActivitiesResult:
        async_result = []
        self._describe_billing_activities(
            request,
            lambda result: async_result.append(result),
            is_blocking=False,
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_billing_activity(
        self,
        request: GetBillingActivityRequest,
        callback: Callable[[AsyncResult[GetBillingActivityResult]], None],
        is_blocking: bool,
    ):
        url = Gs2Constant.ENDPOINT_HOST.format(
            service='watch',
            region=self.session.region,
        ) + "/billingActivity/{year}/{month}/{service}/{activityType}".format(
            year=request.year if request.year is not None and request.year != '' else 'null',
            month=request.month if request.month is not None and request.month != '' else 'null',
            service=request.service if request.service is not None and request.service != '' else 'null',
            activityType=request.activity_type if request.activity_type is not None and request.activity_type != '' else 'null',
        )

        headers = self._create_authorized_headers()
        body = {
            'contextStack': request.context_stack,
        }

        if request.request_id:
            headers["X-GS2-REQUEST-ID"] = request.request_id
        _job = NetworkJob(
            url=url,
            method='POST',
            result_type=GetBillingActivityResult,
            callback=callback,
            headers=headers,
            body=body,
        )

        self.session.send(
            job=_job,
            is_blocking=is_blocking,
        )

    def get_billing_activity(
        self,
        request: GetBillingActivityRequest,
    ) -> GetBillingActivityResult:
        async_result = []
        with timeout(30):
            self._get_billing_activity(
                request,
                lambda result: async_result.append(result),
                is_blocking=True,
            )

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_billing_activity_async(
        self,
        request: GetBillingActivityRequest,
    ) -> GetBillingActivityResult:
        async_result = []
        self._get_billing_activity(
            request,
            lambda result: async_result.append(result),
            is_blocking=False,
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from key import Gs2KeyRestClient, request as request_, result as result_
from key.domain.iterator.namespaces import DescribeNamespacesIterator
from key.domain.iterator.keys import DescribeKeysIterator
from key.domain.iterator.git_hub_api_keys import DescribeGitHubApiKeysIterator
from key.domain.cache.namespace import NamespaceDomainCache
from key.domain.cache.key import KeyDomainCache
from key.domain.cache.git_hub_api_key import GitHubApiKeyDomainCache


class KeyDomain:
    _session: Gs2RestSession
    _client: Gs2KeyRestClient
    _key_cache: KeyDomainCache
    _namespace_name: str
    _key_name: str

    def __init__(
        self,
        session: Gs2RestSession,
        key_cache: KeyDomainCache,
        namespace_name: str,
        key_name: str,
    ):
        self._session = session
        self._client = Gs2KeyRestClient(
            session,
        )
        self._key_cache = key_cache
        self._namespace_name = namespace_name
        self._key_name = key_name

    def update(
        self,
        request: request_.UpdateKeyRequest,
    ) -> result_.UpdateKeyResult:
        request.with_namespace_name(self._namespace_name)
        request.with_key_name(self._key_name)
        r = self._client.update_key(
            request,
        )
        self._key_cache.update(r.item)
        return r

    def load(
        self,
        request: request_.GetKeyRequest,
    ) -> result_.GetKeyResult:
        request.with_namespace_name(self._namespace_name)
        request.with_key_name(self._key_name)
        r = self._client.get_key(
            request,
        )
        self._key_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteKeyRequest,
    ) -> result_.DeleteKeyResult:
        request.with_namespace_name(self._namespace_name)
        request.with_key_name(self._key_name)
        r = self._client.delete_key(
            request,
        )
        self._key_cache.delete(r.item)
        return r

    def encrypt(
        self,
        request: request_.EncryptRequest,
    ) -> result_.EncryptResult:
        request.with_namespace_name(self._namespace_name)
        request.with_key_name(self._key_name)
        r = self._client.encrypt(
            request,
        )
        return r

    def decrypt(
        self,
        request: request_.DecryptRequest,
    ) -> result_.DecryptResult:
        request.with_namespace_name(self._namespace_name)
        request.with_key_name(self._key_name)
        r = self._client.decrypt(
            request,
        )
        return r



# encoding: utf-8
#
# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.

import time
from core.web_socket import *
from core.model import Gs2Constant
from watch.request import *
from watch.result import *


class Gs2WatchWebSocketClient(AbstractGs2WebSocketClient):

    def _get_chart(
        self,
        request: GetChartRequest,
        callback: Callable[[AsyncResult[GetChartResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="watch",
            component='chart',
            function='getChart',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.metrics is not None:
            body["metrics"] = request.metrics
        if request.grn is not None:
            body["grn"] = request.grn
        if request.queries is not None:
            body["queries"] = [
                item
                for item in request.queries
            ]
        if request.by is not None:
            body["by"] = request.by
        if request.timeframe is not None:
            body["timeframe"] = request.timeframe
        if request.size is not None:
            body["size"] = request.size
        if request.format is not None:
            body["format"] = request.format
        if request.aggregator is not None:
            body["aggregator"] = request.aggregator
        if request.style is not None:
            body["style"] = request.style
        if request.title is not None:
            body["title"] = request.title

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            NetworkJob(
                request_id=request_id,
                result_type=GetChartResult,
                callback=callback,
                body=body,
            )
        )

    def get_chart(
        self,
        request: GetChartRequest,
    ) -> GetChartResult:
        async_result = []
        with timeout(30):
            self._get_chart(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_chart_async(
        self,
        request: GetChartRequest,
    ) -> GetChartResult:
        async_result = []
        self._get_chart(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_cumulative(
        self,
        request: GetCumulativeRequest,
        callback: Callable[[AsyncResult[GetCumulativeResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="watch",
            component='cumulative',
            function='getCumulative',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.name is not None:
            body["name"] = request.name
        if request.resource_grn is not None:
            body["resourceGrn"] = request.resource_grn

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            NetworkJob(
                request_id=request_id,
                result_type=GetCumulativeResult,
                callback=callback,
                body=body,
            )
        )

    def get_cumulative(
        self,
        request: GetCumulativeRequest,
    ) -> GetCumulativeResult:
        async_result = []
        with timeout(30):
            self._get_cumulative(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_cumulative_async(
        self,
        request: GetCumulativeRequest,
    ) -> GetCumulativeResult:
        async_result = []
        self._get_cumulative(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _describe_billing_activities(
        self,
        request: DescribeBillingActivitiesRequest,
        callback: Callable[[AsyncResult[DescribeBillingActivitiesResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="watch",
            component='billingActivity',
            function='describeBillingActivities',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.year is not None:
            body["year"] = request.year
        if request.month is not None:
            body["month"] = request.month
        if request.service is not None:
            body["service"] = request.service
        if request.page_token is not None:
            body["pageToken"] = request.page_token
        if request.limit is not None:
            body["limit"] = request.limit

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            NetworkJob(
                request_id=request_id,
                result_type=DescribeBillingActivitiesResult,
                callback=callback,
                body=body,
            )
        )

    def describe_billing_activities(
        self,
        request: DescribeBillingActivitiesRequest,
    ) -> DescribeBillingActivitiesResult:
        async_result = []
        with timeout(30):
            self._describe_billing_activities(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def describe_billing_activities_async(
        self,
        request: DescribeBillingActivitiesRequest,
    ) -> DescribeBillingActivitiesResult:
        async_result = []
        self._describe_billing_activities(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result

    def _get_billing_activity(
        self,
        request: GetBillingActivityRequest,
        callback: Callable[[AsyncResult[GetBillingActivityResult]], None],
    ):
        import uuid

        request_id = str(uuid.uuid4())
        body = self._create_metadata(
            service="watch",
            component='billingActivity',
            function='getBillingActivity',
            request_id=request_id,
        )

        if request.context_stack:
            body['contextStack'] = str(request.context_stack)
        if request.year is not None:
            body["year"] = request.year
        if request.month is not None:
            body["month"] = request.month
        if request.service is not None:
            body["service"] = request.service
        if request.activity_type is not None:
            body["activityType"] = request.activity_type

        if request.request_id:
            body["xGs2RequestId"] = request.request_id

        self.session.send(
            NetworkJob(
                request_id=request_id,
                result_type=GetBillingActivityResult,
                callback=callback,
                body=body,
            )
        )

    def get_billing_activity(
        self,
        request: GetBillingActivityRequest,
    ) -> GetBillingActivityResult:
        async_result = []
        with timeout(30):
            self._get_billing_activity(
                request,
                lambda result: async_result.append(result),
            )

        with timeout(30):
            while not async_result:
                time.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


    async def get_billing_activity_async(
        self,
        request: GetBillingActivityRequest,
    ) -> GetBillingActivityResult:
        async_result = []
        self._get_billing_activity(
            request,
            lambda result: async_result.append(result),
        )

        import asyncio
        with timeout(30):
            while not async_result:
                await asyncio.sleep(0.01)

        if async_result[0].error:
            raise async_result[0].error
        return async_result[0].result


# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from lottery import Gs2LotteryRestClient, request as request_, result as result_
from lottery.domain.iterator.namespaces import DescribeNamespacesIterator
from lottery.domain.iterator.lottery_model_masters import DescribeLotteryModelMastersIterator
from lottery.domain.iterator.prize_table_masters import DescribePrizeTableMastersIterator
from lottery.domain.iterator.boxes import DescribeBoxesIterator
from lottery.domain.iterator.boxes_by_user_id import DescribeBoxesByUserIdIterator
from lottery.domain.iterator.lottery_models import DescribeLotteryModelsIterator
from lottery.domain.iterator.prize_tables import DescribePrizeTablesIterator
from lottery.domain.iterator.probabilities import DescribeProbabilitiesIterator
from lottery.domain.iterator.probabilities_by_user_id import DescribeProbabilitiesByUserIdIterator
from lottery.domain.cache.namespace import NamespaceDomainCache
from lottery.domain.cache.lottery_model_master import LotteryModelMasterDomainCache
from lottery.domain.cache.prize_table_master import PrizeTableMasterDomainCache
from lottery.domain.cache.box import BoxDomainCache
from lottery.domain.cache.lottery_model import LotteryModelDomainCache
from lottery.domain.cache.prize_table import PrizeTableDomainCache
from lottery.domain.cache.probability import ProbabilityDomainCache
from lottery.domain.box import BoxDomain
from lottery.domain.box_access_token import BoxAccessTokenDomain
from lottery.domain.box_access_token import BoxAccessTokenDomain


class UserDomain:
    _session: Gs2RestSession
    _client: Gs2LotteryRestClient
    _namespace_name: str
    _user_id: str
    _box_cache: BoxDomainCache
    _probability_cache: ProbabilityDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_name: str,
        user_id: str,
    ):
        self._session = session
        self._client = Gs2LotteryRestClient(
            session,
        )
        self._namespace_name = namespace_name
        self._user_id = user_id
        self._box_cache = BoxDomainCache()
        self._probability_cache = ProbabilityDomainCache()

    def draw(
        self,
        request: request_.DrawByUserIdRequest,
    ) -> result_.DrawByUserIdResult:
        request.with_namespace_name(self._namespace_name)
        request.with_user_id(self._user_id)
        r = self._client.draw_by_user_id(
            request,
        )
        return r

    def boxes(
        self,
    ) -> DescribeBoxesByUserIdIterator:
        return DescribeBoxesByUserIdIterator(
            self._box_cache,
            self._client,
            self._namespace_name,
            self._user_id,
        )

    def probabilities(
        self,
        lottery_name: str,
    ) -> DescribeProbabilitiesByUserIdIterator:
        return DescribeProbabilitiesByUserIdIterator(
            self._probability_cache,
            self._client,
            self._namespace_name,
            lottery_name,
            self._user_id,
        )

    def box(
        self,
        prize_table_name: str,
    ) -> BoxDomain:
        return BoxDomain(
            self._session,
            self._box_cache,
            self._namespace_name,
            self._user_id,
            prize_table_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from news import Gs2NewsRestClient, request as request_, result as result_
from news.domain.iterator.namespaces import DescribeNamespacesIterator
from news.domain.iterator.news import DescribeNewsIterator
from news.domain.iterator.news_by_user_id import DescribeNewsByUserIdIterator
from news.domain.cache.namespace import NamespaceDomainCache
from news.domain.cache.news import NewsDomainCache
from news.domain.current_news_master import CurrentNewsMasterDomain
from news.domain.user import UserDomain
from news.domain.user_access_token import UserAccessTokenDomain
from news.domain.user_access_token import UserAccessTokenDomain


class NamespaceDomain:
    _session: Gs2RestSession
    _client: Gs2NewsRestClient
    _namespace_cache: NamespaceDomainCache
    _namespace_name: str

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_cache: NamespaceDomainCache,
        namespace_name: str,
    ):
        self._session = session
        self._client = Gs2NewsRestClient(
            session,
        )
        self._namespace_cache = namespace_cache
        self._namespace_name = namespace_name

    def get_status(
        self,
        request: request_.GetNamespaceStatusRequest,
    ) -> result_.GetNamespaceStatusResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace_status(
            request,
        )
        return r

    def load(
        self,
        request: request_.GetNamespaceRequest,
    ) -> result_.GetNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateNamespaceRequest,
    ) -> result_.UpdateNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.update_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteNamespaceRequest,
    ) -> result_.DeleteNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.delete_namespace(
            request,
        )
        return r

    def current_news_master(
        self,
    ) -> CurrentNewsMasterDomain:
        return CurrentNewsMasterDomain(
            self._session,
            self._namespace_name,
        )

    def user(
        self,
        user_id: str,
    ) -> UserDomain:
        return UserDomain(
            self._session,
            self._namespace_name,
            user_id,
        )

    def access_token(
        self,
        access_token: AccessToken,
    ) -> UserAccessTokenDomain:
        return UserAccessTokenDomain(
            self._session,
            self._namespace_name,
            access_token,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from script import Gs2ScriptRestClient, request as request_, result as result_
from script.domain.iterator.namespaces import DescribeNamespacesIterator
from script.domain.cache.namespace import NamespaceDomainCache
from script.domain.namespace import NamespaceDomain

class Gs2Script:
    _session: Gs2RestSession
    _client: Gs2ScriptRestClient
    _namespace_cache: NamespaceDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
    ):
        self._session = session
        self._client = Gs2ScriptRestClient (
            session,
        )
        self._namespace_cache = NamespaceDomainCache()

    def create_namespace(
        self,
        request: request_.CreateNamespaceRequest,
    ) -> result_.CreateNamespaceResult:
        r = self._client.create_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def invoke_script(
        self,
        request: request_.InvokeScriptRequest,
    ) -> result_.InvokeScriptResult:
        r = self._client.invoke_script(
            request,
        )
        return r

    def debug_invoke(
        self,
        request: request_.DebugInvokeRequest,
    ) -> result_.DebugInvokeResult:
        r = self._client.debug_invoke(
            request,
        )
        return r

    def namespaces(
        self,
    ) -> DescribeNamespacesIterator:
        return DescribeNamespacesIterator(
            self._namespace_cache,
            self._client,
        )

    def namespace(
        self,
        namespace_name: str,
    ) -> NamespaceDomain:
        return NamespaceDomain(
            self._session,
            self._namespace_cache,
            namespace_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from showcase import Gs2ShowcaseRestClient, request as request_, result as result_
from showcase.domain.iterator.namespaces import DescribeNamespacesIterator
from showcase.domain.iterator.sales_item_masters import DescribeSalesItemMastersIterator
from showcase.domain.iterator.sales_item_group_masters import DescribeSalesItemGroupMastersIterator
from showcase.domain.iterator.showcase_masters import DescribeShowcaseMastersIterator
from showcase.domain.iterator.showcases import DescribeShowcasesIterator
from showcase.domain.iterator.showcases_by_user_id import DescribeShowcasesByUserIdIterator
from showcase.domain.cache.namespace import NamespaceDomainCache
from showcase.domain.cache.sales_item_master import SalesItemMasterDomainCache
from showcase.domain.cache.sales_item_group_master import SalesItemGroupMasterDomainCache
from showcase.domain.cache.showcase_master import ShowcaseMasterDomainCache
from showcase.domain.cache.showcase import ShowcaseDomainCache
from showcase.domain.showcase import ShowcaseDomain
from showcase.domain.showcase_access_token import ShowcaseAccessTokenDomain
from showcase.domain.showcase_access_token import ShowcaseAccessTokenDomain


class UserDomain:
    _session: Gs2RestSession
    _client: Gs2ShowcaseRestClient
    _namespace_name: str
    _user_id: str
    _showcase_cache: ShowcaseDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_name: str,
        user_id: str,
    ):
        self._session = session
        self._client = Gs2ShowcaseRestClient(
            session,
        )
        self._namespace_name = namespace_name
        self._user_id = user_id
        self._showcase_cache = ShowcaseDomainCache()

    def showcases(
        self,
    ) -> DescribeShowcasesByUserIdIterator:
        return DescribeShowcasesByUserIdIterator(
            self._showcase_cache,
            self._client,
            self._namespace_name,
            self._user_id,
        )

    def showcase(
        self,
        showcase_name: str,
    ) -> ShowcaseDomain:
        return ShowcaseDomain(
            self._session,
            self._showcase_cache,
            self._namespace_name,
            self._user_id,
            showcase_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from matchmaking import Gs2MatchmakingRestClient, request as request_, result as result_
from matchmaking.domain.iterator.namespaces import DescribeNamespacesIterator
from matchmaking.domain.iterator.gatherings import DescribeGatheringsIterator
from matchmaking.domain.iterator.rating_model_masters import DescribeRatingModelMastersIterator
from matchmaking.domain.iterator.rating_models import DescribeRatingModelsIterator
from matchmaking.domain.iterator.ratings import DescribeRatingsIterator
from matchmaking.domain.iterator.ratings_by_user_id import DescribeRatingsByUserIdIterator
from matchmaking.domain.cache.namespace import NamespaceDomainCache
from matchmaking.domain.cache.gathering import GatheringDomainCache
from matchmaking.domain.cache.rating_model_master import RatingModelMasterDomainCache
from matchmaking.domain.cache.rating_model import RatingModelDomainCache
from matchmaking.domain.cache.rating import RatingDomainCache
from matchmaking.domain.rating import RatingDomain
from matchmaking.domain.rating_access_token import RatingAccessTokenDomain
from matchmaking.domain.rating_access_token import RatingAccessTokenDomain
from matchmaking.domain.gathering import GatheringDomain
from matchmaking.domain.gathering_access_token import GatheringAccessTokenDomain
from matchmaking.domain.gathering_access_token import GatheringAccessTokenDomain
from matchmaking.domain.vote import VoteDomain
from matchmaking.domain.vote_access_token import VoteAccessTokenDomain
from matchmaking.domain.vote_access_token import VoteAccessTokenDomain


class UserAccessTokenDomain:
    _session: Gs2RestSession
    _client: Gs2MatchmakingRestClient
    _namespace_name: str
    _access_token: AccessToken
    _gathering_cache: GatheringDomainCache
    _rating_cache: RatingDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_name: str,
        access_token: AccessToken,
    ):
        self._session = session
        self._client = Gs2MatchmakingRestClient(
            session,
        )
        self._namespace_name = namespace_name
        self._access_token = access_token
        self._gathering_cache = GatheringDomainCache()
        self._rating_cache = RatingDomainCache()

    def create_gathering(
        self,
        request: request_.CreateGatheringRequest,
    ) -> result_.CreateGatheringResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        r = self._client.create_gathering(
            request,
        )
        return r

    def do_matchmaking(
        self,
        request: request_.DoMatchmakingRequest,
    ) -> result_.DoMatchmakingResult:
        request.with_namespace_name(self._namespace_name)
        request.with_access_token(self._access_token.token if self._access_token else None)
        r = self._client.do_matchmaking(
            request,
        )
        return r

    def ratings(
        self,
    ) -> DescribeRatingsIterator:
        return DescribeRatingsIterator(
            self._rating_cache,
            self._client,
            self._namespace_name,
            self._access_token,
        )

    def rating(
        self,
        rating_name: str,
    ) -> RatingAccessTokenDomain:
        return RatingAccessTokenDomain(
            self._session,
            self._rating_cache,
            self._namespace_name,
            self._access_token,
            rating_name,
        )

    def gathering(
        self,
        gathering_name: str,
    ) -> GatheringAccessTokenDomain:
        return GatheringAccessTokenDomain(
            self._session,
            self._gathering_cache,
            self._namespace_name,
            self._access_token,
            gathering_name,
        )

    def vote(
        self,
        rating_name: str,
        gathering_name: str,
    ) -> VoteAccessTokenDomain:
        return VoteAccessTokenDomain(
            self._session,
            self._namespace_name,
            self._access_token,
            rating_name,
            gathering_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from chat import Gs2ChatRestClient, request as request_, result as result_
from chat.domain.iterator.namespaces import DescribeNamespacesIterator
from chat.domain.iterator.rooms import DescribeRoomsIterator
from chat.domain.iterator.messages import DescribeMessagesIterator
from chat.domain.iterator.messages_by_user_id import DescribeMessagesByUserIdIterator
from chat.domain.iterator.subscribes import DescribeSubscribesIterator
from chat.domain.iterator.subscribes_by_user_id import DescribeSubscribesByUserIdIterator
from chat.domain.iterator.subscribes_by_room_name import DescribeSubscribesByRoomNameIterator
from chat.domain.cache.namespace import NamespaceDomainCache
from chat.domain.cache.room import RoomDomainCache
from chat.domain.cache.message import MessageDomainCache
from chat.domain.cache.subscribe import SubscribeDomainCache


class SubscribeDomain:
    _session: Gs2RestSession
    _client: Gs2ChatRestClient
    _subscribe_cache: SubscribeDomainCache
    _namespace_name: str
    _user_id: str
    _room_name: str

    def __init__(
        self,
        session: Gs2RestSession,
        subscribe_cache: SubscribeDomainCache,
        namespace_name: str,
        user_id: str,
        room_name: str,
    ):
        self._session = session
        self._client = Gs2ChatRestClient(
            session,
        )
        self._subscribe_cache = subscribe_cache
        self._namespace_name = namespace_name
        self._user_id = user_id
        self._room_name = room_name

    def subscribe(
        self,
        request: request_.SubscribeByUserIdRequest,
    ) -> result_.SubscribeByUserIdResult:
        request.with_namespace_name(self._namespace_name)
        request.with_user_id(self._user_id)
        request.with_room_name(self._room_name)
        r = self._client.subscribe_by_user_id(
            request,
        )
        self._subscribe_cache.update(r.item)
        return r

    def load(
        self,
        request: request_.GetSubscribeByUserIdRequest,
    ) -> result_.GetSubscribeByUserIdResult:
        request.with_namespace_name(self._namespace_name)
        request.with_user_id(self._user_id)
        request.with_room_name(self._room_name)
        r = self._client.get_subscribe_by_user_id(
            request,
        )
        self._subscribe_cache.update(r.item)
        return r

    def update_notification_type(
        self,
        request: request_.UpdateNotificationTypeByUserIdRequest,
    ) -> result_.UpdateNotificationTypeByUserIdResult:
        request.with_namespace_name(self._namespace_name)
        request.with_user_id(self._user_id)
        request.with_room_name(self._room_name)
        r = self._client.update_notification_type_by_user_id(
            request,
        )
        self._subscribe_cache.update(r.item)
        return r

    def unsubscribe(
        self,
        request: request_.UnsubscribeByUserIdRequest,
    ) -> result_.UnsubscribeByUserIdResult:
        request.with_namespace_name(self._namespace_name)
        request.with_user_id(self._user_id)
        request.with_room_name(self._room_name)
        r = self._client.unsubscribe_by_user_id(
            request,
        )
        self._subscribe_cache.update(r.item)
        return r



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from stamina import Gs2StaminaRestClient, request as request_, result as result_
from stamina.domain.iterator.namespaces import DescribeNamespacesIterator
from stamina.domain.iterator.stamina_model_masters import DescribeStaminaModelMastersIterator
from stamina.domain.iterator.max_stamina_table_masters import DescribeMaxStaminaTableMastersIterator
from stamina.domain.iterator.recover_interval_table_masters import DescribeRecoverIntervalTableMastersIterator
from stamina.domain.iterator.recover_value_table_masters import DescribeRecoverValueTableMastersIterator
from stamina.domain.iterator.stamina_models import DescribeStaminaModelsIterator
from stamina.domain.iterator.staminas import DescribeStaminasIterator
from stamina.domain.iterator.staminas_by_user_id import DescribeStaminasByUserIdIterator
from stamina.domain.cache.namespace import NamespaceDomainCache
from stamina.domain.cache.stamina_model_master import StaminaModelMasterDomainCache
from stamina.domain.cache.max_stamina_table_master import MaxStaminaTableMasterDomainCache
from stamina.domain.cache.recover_interval_table_master import RecoverIntervalTableMasterDomainCache
from stamina.domain.cache.recover_value_table_master import RecoverValueTableMasterDomainCache
from stamina.domain.cache.stamina_model import StaminaModelDomainCache
from stamina.domain.cache.stamina import StaminaDomainCache


class StaminaModelMasterDomain:
    _session: Gs2RestSession
    _client: Gs2StaminaRestClient
    _stamina_model_master_cache: StaminaModelMasterDomainCache
    _namespace_name: str
    _stamina_name: str

    def __init__(
        self,
        session: Gs2RestSession,
        stamina_model_master_cache: StaminaModelMasterDomainCache,
        namespace_name: str,
        stamina_name: str,
    ):
        self._session = session
        self._client = Gs2StaminaRestClient(
            session,
        )
        self._stamina_model_master_cache = stamina_model_master_cache
        self._namespace_name = namespace_name
        self._stamina_name = stamina_name

    def load(
        self,
        request: request_.GetStaminaModelMasterRequest,
    ) -> result_.GetStaminaModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        request.with_stamina_name(self._stamina_name)
        r = self._client.get_stamina_model_master(
            request,
        )
        self._stamina_model_master_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateStaminaModelMasterRequest,
    ) -> result_.UpdateStaminaModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        request.with_stamina_name(self._stamina_name)
        r = self._client.update_stamina_model_master(
            request,
        )
        self._stamina_model_master_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteStaminaModelMasterRequest,
    ) -> result_.DeleteStaminaModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        request.with_stamina_name(self._stamina_name)
        r = self._client.delete_stamina_model_master(
            request,
        )
        self._stamina_model_master_cache.delete(r.item)
        return r



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from version import Gs2VersionRestClient, request as request_, result as result_
from version.domain.iterator.namespaces import DescribeNamespacesIterator
from version.domain.iterator.version_model_masters import DescribeVersionModelMastersIterator
from version.domain.iterator.version_models import DescribeVersionModelsIterator
from version.domain.iterator.accept_versions import DescribeAcceptVersionsIterator
from version.domain.iterator.accept_versions_by_user_id import DescribeAcceptVersionsByUserIdIterator
from version.domain.cache.namespace import NamespaceDomainCache
from version.domain.cache.version_model_master import VersionModelMasterDomainCache
from version.domain.cache.version_model import VersionModelDomainCache
from version.domain.cache.accept_version import AcceptVersionDomainCache
from version.domain.current_version_master import CurrentVersionMasterDomain
from version.domain.version_model import VersionModelDomain
from version.domain.user import UserDomain
from version.domain.user_access_token import UserAccessTokenDomain
from version.domain.user_access_token import UserAccessTokenDomain
from version.domain.version_model_master import VersionModelMasterDomain


class NamespaceDomain:
    _session: Gs2RestSession
    _client: Gs2VersionRestClient
    _namespace_cache: NamespaceDomainCache
    _namespace_name: str
    _version_model_master_cache: VersionModelMasterDomainCache
    _version_model_cache: VersionModelDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_cache: NamespaceDomainCache,
        namespace_name: str,
    ):
        self._session = session
        self._client = Gs2VersionRestClient(
            session,
        )
        self._namespace_cache = namespace_cache
        self._namespace_name = namespace_name
        self._version_model_master_cache = VersionModelMasterDomainCache()
        self._version_model_cache = VersionModelDomainCache()

    def get_status(
        self,
        request: request_.GetNamespaceStatusRequest,
    ) -> result_.GetNamespaceStatusResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace_status(
            request,
        )
        return r

    def load(
        self,
        request: request_.GetNamespaceRequest,
    ) -> result_.GetNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateNamespaceRequest,
    ) -> result_.UpdateNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.update_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteNamespaceRequest,
    ) -> result_.DeleteNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.delete_namespace(
            request,
        )
        self._namespace_cache.delete(r.item)
        return r

    def create_version_model_master(
        self,
        request: request_.CreateVersionModelMasterRequest,
    ) -> result_.CreateVersionModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_version_model_master(
            request,
        )
        return r

    def version_model_masters(
        self,
    ) -> DescribeVersionModelMastersIterator:
        return DescribeVersionModelMastersIterator(
            self._version_model_master_cache,
            self._client,
            self._namespace_name,
        )

    def version_models(
        self,
    ) -> DescribeVersionModelsIterator:
        return DescribeVersionModelsIterator(
            self._version_model_cache,
            self._client,
            self._namespace_name,
        )

    def current_version_master(
        self,
    ) -> CurrentVersionMasterDomain:
        return CurrentVersionMasterDomain(
            self._session,
            self._namespace_name,
        )

    def version_model(
        self,
        version_name: str,
    ) -> VersionModelDomain:
        return VersionModelDomain(
            self._session,
            self._version_model_cache,
            self._namespace_name,
            version_name,
        )

    def user(
        self,
        user_id: str,
    ) -> UserDomain:
        return UserDomain(
            self._session,
            self._namespace_name,
            user_id,
        )

    def access_token(
        self,
        access_token: AccessToken,
    ) -> UserAccessTokenDomain:
        return UserAccessTokenDomain(
            self._session,
            self._namespace_name,
            access_token,
        )

    def version_model_master(
        self,
        version_name: str,
    ) -> VersionModelMasterDomain:
        return VersionModelMasterDomain(
            self._session,
            self._version_model_master_cache,
            self._namespace_name,
            version_name,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from gateway import Gs2GatewayRestClient, request as request_, result as result_
from gateway.domain.iterator.namespaces import DescribeNamespacesIterator
from gateway.domain.iterator.web_socket_sessions import DescribeWebSocketSessionsIterator
from gateway.domain.iterator.web_socket_sessions_by_user_id import DescribeWebSocketSessionsByUserIdIterator
from gateway.domain.cache.namespace import NamespaceDomainCache
from gateway.domain.cache.web_socket_session import WebSocketSessionDomainCache
from gateway.domain.web_socket_session import WebSocketSessionDomain
from gateway.domain.web_socket_session_access_token import WebSocketSessionAccessTokenDomain
from gateway.domain.web_socket_session_access_token import WebSocketSessionAccessTokenDomain
from gateway.domain.firebase_token import FirebaseTokenDomain
from gateway.domain.firebase_token_access_token import FirebaseTokenAccessTokenDomain
from gateway.domain.firebase_token_access_token import FirebaseTokenAccessTokenDomain


class UserDomain:
    _session: Gs2RestSession
    _client: Gs2GatewayRestClient
    _namespace_name: str
    _user_id: str
    _web_socket_session_cache: WebSocketSessionDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_name: str,
        user_id: str,
    ):
        self._session = session
        self._client = Gs2GatewayRestClient(
            session,
        )
        self._namespace_name = namespace_name
        self._user_id = user_id
        self._web_socket_session_cache = WebSocketSessionDomainCache()

    def web_socket_sessions(
        self,
    ) -> DescribeWebSocketSessionsByUserIdIterator:
        return DescribeWebSocketSessionsByUserIdIterator(
            self._web_socket_session_cache,
            self._client,
            self._namespace_name,
            self._user_id,
        )

    def web_socket_session(
        self,
    ) -> WebSocketSessionDomain:
        return WebSocketSessionDomain(
            self._session,
            self._web_socket_session_cache,
            self._namespace_name,
            self._user_id,
        )

    def firebase_token(
        self,
    ) -> FirebaseTokenDomain:
        return FirebaseTokenDomain(
            self._session,
            self._namespace_name,
            self._user_id,
        )



# Copyright 2016 Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations

from core import Gs2RestSession
from core.domain.access_token import AccessToken
from stamina import Gs2StaminaRestClient, request as request_, result as result_
from stamina.domain.iterator.namespaces import DescribeNamespacesIterator
from stamina.domain.iterator.stamina_model_masters import DescribeStaminaModelMastersIterator
from stamina.domain.iterator.max_stamina_table_masters import DescribeMaxStaminaTableMastersIterator
from stamina.domain.iterator.recover_interval_table_masters import DescribeRecoverIntervalTableMastersIterator
from stamina.domain.iterator.recover_value_table_masters import DescribeRecoverValueTableMastersIterator
from stamina.domain.iterator.stamina_models import DescribeStaminaModelsIterator
from stamina.domain.iterator.staminas import DescribeStaminasIterator
from stamina.domain.iterator.staminas_by_user_id import DescribeStaminasByUserIdIterator
from stamina.domain.cache.namespace import NamespaceDomainCache
from stamina.domain.cache.stamina_model_master import StaminaModelMasterDomainCache
from stamina.domain.cache.max_stamina_table_master import MaxStaminaTableMasterDomainCache
from stamina.domain.cache.recover_interval_table_master import RecoverIntervalTableMasterDomainCache
from stamina.domain.cache.recover_value_table_master import RecoverValueTableMasterDomainCache
from stamina.domain.cache.stamina_model import StaminaModelDomainCache
from stamina.domain.cache.stamina import StaminaDomainCache
from stamina.domain.current_stamina_master import CurrentStaminaMasterDomain
from stamina.domain.stamina_model import StaminaModelDomain
from stamina.domain.user import UserDomain
from stamina.domain.user_access_token import UserAccessTokenDomain
from stamina.domain.user_access_token import UserAccessTokenDomain
from stamina.domain.recover_interval_table_master import RecoverIntervalTableMasterDomain
from stamina.domain.max_stamina_table_master import MaxStaminaTableMasterDomain
from stamina.domain.recover_value_table_master import RecoverValueTableMasterDomain
from stamina.domain.stamina_model_master import StaminaModelMasterDomain


class NamespaceDomain:
    _session: Gs2RestSession
    _client: Gs2StaminaRestClient
    _namespace_cache: NamespaceDomainCache
    _namespace_name: str
    _stamina_model_master_cache: StaminaModelMasterDomainCache
    _max_stamina_table_master_cache: MaxStaminaTableMasterDomainCache
    _recover_interval_table_master_cache: RecoverIntervalTableMasterDomainCache
    _recover_value_table_master_cache: RecoverValueTableMasterDomainCache
    _stamina_model_cache: StaminaModelDomainCache

    def __init__(
        self,
        session: Gs2RestSession,
        namespace_cache: NamespaceDomainCache,
        namespace_name: str,
    ):
        self._session = session
        self._client = Gs2StaminaRestClient(
            session,
        )
        self._namespace_cache = namespace_cache
        self._namespace_name = namespace_name
        self._stamina_model_master_cache = StaminaModelMasterDomainCache()
        self._max_stamina_table_master_cache = MaxStaminaTableMasterDomainCache()
        self._recover_interval_table_master_cache = RecoverIntervalTableMasterDomainCache()
        self._recover_value_table_master_cache = RecoverValueTableMasterDomainCache()
        self._stamina_model_cache = StaminaModelDomainCache()

    def get_status(
        self,
        request: request_.GetNamespaceStatusRequest,
    ) -> result_.GetNamespaceStatusResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace_status(
            request,
        )
        return r

    def load(
        self,
        request: request_.GetNamespaceRequest,
    ) -> result_.GetNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.get_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def update(
        self,
        request: request_.UpdateNamespaceRequest,
    ) -> result_.UpdateNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.update_namespace(
            request,
        )
        self._namespace_cache.update(r.item)
        return r

    def delete(
        self,
        request: request_.DeleteNamespaceRequest,
    ) -> result_.DeleteNamespaceResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.delete_namespace(
            request,
        )
        self._namespace_cache.delete(r.item)
        return r

    def create_stamina_model_master(
        self,
        request: request_.CreateStaminaModelMasterRequest,
    ) -> result_.CreateStaminaModelMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_stamina_model_master(
            request,
        )
        return r

    def create_max_stamina_table_master(
        self,
        request: request_.CreateMaxStaminaTableMasterRequest,
    ) -> result_.CreateMaxStaminaTableMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_max_stamina_table_master(
            request,
        )
        return r

    def create_recover_interval_table_master(
        self,
        request: request_.CreateRecoverIntervalTableMasterRequest,
    ) -> result_.CreateRecoverIntervalTableMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_recover_interval_table_master(
            request,
        )
        return r

    def create_recover_value_table_master(
        self,
        request: request_.CreateRecoverValueTableMasterRequest,
    ) -> result_.CreateRecoverValueTableMasterResult:
        request.with_namespace_name(self._namespace_name)
        r = self._client.create_recover_value_table_master(
            request,
        )
        return r

    def stamina_model_masters(
        self,
    ) -> DescribeStaminaModelMastersIterator:
        return DescribeStaminaModelMastersIterator(
            self._stamina_model_master_cache,
            self._client,
            self._namespace_name,
        )

    def max_stamina_table_masters(
        self,
    ) -> DescribeMaxStaminaTableMastersIterator:
        return DescribeMaxStaminaTableMastersIterator(
            self._max_stamina_table_master_cache,
            self._client,
            self._namespace_name,
        )

    def recover_interval_table_masters(
        self,
    ) -> DescribeRecoverIntervalTableMastersIterator:
        return DescribeRecoverIntervalTableMastersIterator(
            self._recover_interval_table_master_cache,
            self._client,
            self._namespace_name,
        )

    def recover_value_table_masters(
        self,
    ) -> DescribeRecoverValueTableMastersIterator:
        return DescribeRecoverValueTableMastersIterator(
            self._recover_value_table_master_cache,
            self._client,
            self._namespace_name,
        )

    def stamina_models(
        self,
    ) -> DescribeStaminaModelsIterator:
        return DescribeStaminaModelsIterator(
            self._stamina_model_cache,
            self._client,
            self._namespace_name,
        )

    def current_stamina_master(
        self,
    ) -> CurrentStaminaMasterDomain:
        return CurrentStaminaMasterDomain(
            self._session,
            self._namespace_name,
        )

    def stamina_model(
        self,
        stamina_name: str,
    ) -> StaminaModelDomain:
        return StaminaModelDomain(
            self._session,
            self._stamina_model_cache,
            self._namespace_name,
            stamina_name,
        )

    def user(
        self,
        user_id: str,
    ) -> UserDomain:
        return UserDomain(
            self._session,
            self._namespace_name,
            user_id,
        )

    def access_token(
        self,
        access_token: AccessToken,
    ) -> UserAccessTokenDomain:
        return UserAccessTokenDomain(
            self._session,
            self._namespace_name,
            access_token,
        )

    def recover_interval_table_master(
        self,
        recover_interval_table_name: str,
    ) -> RecoverIntervalTableMasterDomain:
        return RecoverIntervalTableMasterDomain(
            self._session,
            self._recover_interval_table_master_cache,
            self._namespace_name,
            recover_interval_table_name,
        )

    def max_stamina_table_master(
        self,
        max_stamina_table_name: str,
    ) -> MaxStaminaTableMasterDomain:
        return MaxStaminaTableMasterDomain(
            self._session,
            self._max_stamina_table_master_cache,
            self._namespace_name,
            max_stamina_table_name,
        )

    def recover_value_table_master(
        self,
        recover_value_table_name: str,
    ) -> RecoverValueTableMasterDomain:
        return RecoverValueTableMasterDomain(
            self._session,
            self._recover_value_table_master_cache,
            self._namespace_name,
            recover_value_table_name,
        )

    def stamina_model_master(
        self,
        stamina_name: str,
    ) -> StaminaModelMasterDomain:
        return StaminaModelMasterDomain(
            self._session,
            self._stamina_model_master_cache,
            self._namespace_name,
            stamina_name,
        )
